// 
// UrsTimeElements.cpp
// 
#include <pgmspace.h>
#include <string.h>
#include <TimeLib.h>
#include <UrsTimeElements.h>

static char buffer[15];  // muss gro genug fr den lnsgten String + \0 sein.

static const char ms0[] PROGMEM = "";
static const char ms1[] PROGMEM = "Januar";
static const char ms2[] PROGMEM = "Februar";
static const char ms3[] PROGMEM = "Mrz";
static const char ms4[] PROGMEM = "April";
static const char ms5[] PROGMEM = "Mai";
static const char ms6[] PROGMEM = "Juni";
static const char ms7[] PROGMEM = "Juli";
static const char ms8[] PROGMEM = "August";
static const char ms9[] PROGMEM = "September";
static const char ms10[] PROGMEM = "Oktober";
static const char ms11[] PROGMEM = "November";
static const char ms12[] PROGMEM = "Dezember";

static const PROGMEM char * const PROGMEM ms_P[] ={ms0,ms1,ms2,ms3,ms4,ms5,ms6,ms7,ms8,ms9,ms10,ms11,ms12};
static const int monthShortNameLength = 3;
static const char monthShortNames_P[] PROGMEM = "ErrJanFebMrAprMaiJunJulAugSepOktNovDez";

static const char dStr0[] PROGMEM = "Err";
static const char dStr1[] PROGMEM = "Sonntag";
static const char dStr2[] PROGMEM = "Montag";
static const char dStr3[] PROGMEM = "Dienstag";
static const char dStr4[] PROGMEM = "Mittwoch";
static const char dStr5[] PROGMEM = "Donnerstag";
static const char dStr6[] PROGMEM = "Freitag";
static const char dStr7[] PROGMEM = "Samstag";

static const PROGMEM char * const PROGMEM dayNames_P[] = {dStr0, dStr1, dStr2, dStr3, dStr4, dStr5, dStr6, dStr7};

static const int dayShortNameLength = 2;
static const char dayShortNames_P[] PROGMEM = "ErSoMoDiMiDoFrSa";


UrsTimeLang UrsTimeElements::defaultLang = UrsTimeLang::de; // Festlegung der Standard-Sprache
void UrsTimeElements::setDefaultLanguage(UrsTimeLang dl) {
  if (dl != UrsTimeLang::standard)
    defaultLang = dl;
}

String UrsTimeElements::defaultFormat = "%Y-%m-%d %H:%M:%S";
void UrsTimeElements::setDefaultFormat(String df) {
  UrsTimeElements::defaultFormat = df;
}

String UrsTimeElements::monthStrDE(const uint8_t month) const {
  return strcpy_P(buffer, (PGM_P)pgm_read_dword(&(ms_P[month])));
}

String UrsTimeElements::monthShortStrDE(const uint8_t month)  const {
  for (int i = 0; i < monthShortNameLength; i++)
    buffer[i] = pgm_read_byte(&(monthShortNames_P[i + (month*monthShortNameLength)]));
  buffer[monthShortNameLength] = 0;
  return buffer;
}

String UrsTimeElements::dayStrDE(const uint8_t day)  const { 
  return strcpy_P(buffer, (PGM_P)pgm_read_dword(&(dayNames_P[day])));
}

String UrsTimeElements::dayShortStrDE(const uint8_t day)  const {
  uint8_t index = day*dayShortNameLength;
  for (int i = 0; i < dayShortNameLength; i++)
    buffer[i] = pgm_read_byte(&(dayShortNames_P[index + i]));
  buffer[dayShortNameLength] = 0;
  return buffer;
}

String UrsTimeElements::monthStr(const UrsTimeLang l) const {
  uint8_t m = month;
  if (m > 12) m = 0;

  UrsTimeLang q = l == UrsTimeLang::standard ? defaultLang: l;

  switch (q) {
  case UrsTimeLang::en:
    return  ::monthStr(m);
  case UrsTimeLang::de:
    return monthStrDE(m);
  default:
    return String();
  }
}

String UrsTimeElements::monthShortStr(const UrsTimeLang l) const {
  uint8_t m = month;
  if (m > 12) m = 0;
 
  UrsTimeLang q = l == UrsTimeLang::standard ? defaultLang : l;

  switch (q) {
  case UrsTimeLang::en:
    return  ::monthShortStr(m);
  case UrsTimeLang::de:
    return monthShortStrDE(m);
  default:
    return String();
  }
}

String UrsTimeElements::dayStr(const UrsTimeLang l) const {
  uint8_t d = wDay;
  if (d > 7) d = 0;

  UrsTimeLang q = l == UrsTimeLang::standard ? defaultLang : l;
  
  switch (q) {
  case UrsTimeLang::en:
    return  ::dayStr(d);
  case UrsTimeLang::de:
    return dayStrDE(d);
  default:
    return String();
  }
}

String UrsTimeElements::dayShortStr(const UrsTimeLang l) const {
  uint8_t d = wDay;
  if (d > 7) d = 0;

  UrsTimeLang q = l == UrsTimeLang::standard ? defaultLang : l;

  switch (q) {
  case UrsTimeLang::en:
    return  ::dayShortStr(d);
  case UrsTimeLang::de:
    return dayShortStrDE(d);
  default:
    return String();
  }
}

String UrsTimeElements::toString(const String format, const UrsTimeLang l) const {
  String result;
  result.reserve(64); // Speicher allokieren um das Speicher-Management zu entlasten
  int t1;
  int t2;

  for (uint8_t i = 0; i < format.length(); i++) {
    if (format[i] == '%') {
      if (++i >= format.length()) return result; // % war das letzte Zeichen
      switch (format[i]) { // nchstes Zeichen nach '%'
      case 'a': // Abgekrzten Namen des Wochentags
        result += dayShortStr(l);
        break;
      case 'A': // Vollen Wochentagsnamen
        result += dayStr(l);
        break;
      case 'b': // Abgekrzte Name des Monats
        result += monthShortStr(l);
        break;
      case 'B': // Vollstndigen Monatsnamen
        result += monthStr(l);
        break;
      case 'd': // Tag des Monats als Dezimalzahl(01 - 31)
        result += (char)(day / 10 + '0');
        result += (char)(day % 10 + '0');
        break;
      case 'H': // Die Stunde im 24 - Stunden - Format(00  23)
        result += (char)(hour / 10 + '0');
        result += (char)(hour % 10 + '0');
        break;
      case 'I': // Stundenangabe im 12 - Stunden - Format(01 - 12)
        t1 = hour; if (t1 > 11) t1 -= 12; if (t1 == 0) t1 = 12;
        result += (char)(t1 / 10 + '0');
        result += (char)(t1 % 10 + '0');
        break;
       case 'm': // Monatsangabe als Dezimalzahl(01 - 12)
        result += (char)(month / 10 + '0');
        result += (char)(month % 10 + '0');
        break;
       case 'M': // Minutenangabe als Dezimalzahl(00  59)
         result += (char)(minute / 10 + '0');
         result += (char)(minute % 10 + '0');         break;
       case 'p': // A.M. / P.M. - Kennung des aktuellen Gebietsschemas.fr 12 - Stunden - Uhr
         if (hour < 12)
           result += F("A.M.");
         else
           result += F("P.M.");
           break;
       case 'S': // Sekundenangabe als Dezimalzahl(00  59)
         result += (char)(second / 10 + '0');
         result += (char)(second % 10 + '0');
         break;
       case 'w': // Angabe des Wochentags als Dezimalzahl(0  6; Sonntag ist 0)
         result += (char)(wDay - 1 + '0');
         break;
      case 'y': // Jahresangabe ohne Jahrhundert als Dezimalzahl(00 - 99)
        t1 = year;
        while (t1 > 100) t1 -= 100;
        result += (char)(t1 / 10 + '0');
        result += (char)(t1 % 10 + '0');
        break;
      case 'Y': // Jahresangabe mit Jahrhundert als Dezimalzahl
        t1 = year; t2 = t1 / 1000; result += (char)(t2 + '0');
        t1 -= 1000 * t2; t2 = t1 / 100; result += (char)(t2 + '0');
        t1 -= 100 * t2; t2 = t1 / 10; result += (char)(t2 + '0');
        result += (char)(t1 % 10 + '0');
        break;
      case 'z':
      case 'Z': // Zeitzonenname oder Abkrzung der Zeitzone, je nach Registrierungseintrge; keine Zeichen, wenn Zeitzone unbekannt ist
        result += TimeZone;
        break;
      case '%': // Prozentzeichen
        result += '%';
        break;
      default:
        result += "%"; result += format[i];
        break;
      }
    }
    else // kein '%'-Zeichen
      result += format[i];
  }

  return result;
}

String UrsTimeElements::toString() const {
  return toString(defaultFormat);
}

size_t UrsTimeElements::printTo(Print & p) const {
  String s = toString(defaultFormat);
  p.print(s);
  return s.length();
}